@@ -4,24 +4,26 @@ import sys
 from pathlib import Path
 
 __package__ = "meld"
-__version__ = "3.21.0"
+__version__ = "3.21.1"
 
 APPLICATION_NAME = 'Meld'
 APPLICATION_ID = 'org.gnome.Meld'
 SETTINGS_SCHEMA_ID = 'org.gnome.meld'
 RESOURCE_BASE = '/org/gnome/meld'
 
-DATADIR = "/usr/local/share/meld"
-LOCALEDIR = "/usr/local/share/locale"
+# START; these paths are clobbered on install by meld.build_helpers
+DATADIR = Path(sys.prefix) / "share" / "meld"
+LOCALEDIR = Path(sys.prefix) / "share" / "locale"
+# END
 
-CONFIGURED = '@configured@'
+CONFIGURED = 'True'
 PROFILE = ''
 
 if CONFIGURED == 'True':
-    APPLICATION_ID = '@application_id@'
-    DATADIR = '@pkgdatadir@'
-    LOCALEDIR = '@localedir@'
-    PROFILE = '@profile@'
+    APPLICATION_ID = 'org.gnome.Meld'
+    DATADIR = '/usr/local/share/meld'
+    LOCALEDIR = '/usr/local/share/locale'
+    PROFILE = ''
 
 # Flag enabling some workarounds if data dir isn't installed in standard prefix
 DATADIR_IS_UNINSTALLED = False
@@ -57,4 +59,4 @@ def uninstalled():
     DATADIR_IS_UNINSTALLED = True
 
     resource_path = melddir / "meld" / "resources"
-    os.environ['G_RESOURCE_OVERLAYS'] = f'{RESOURCE_BASE}={resource_path}'
\ No newline at end of file
+    os.environ['G_RESOURCE_OVERLAYS'] = f'{RESOURCE_BASE}={resource_path}'
