@@ -41,8 +41,8 @@ class MeldApp(Gtk.Application):
 
     def __init__(self):
         super().__init__(
-          application_id=meld.conf.APPLICATION_ID,
-          flags=Gio.ApplicationFlags.HANDLES_COMMAND_LINE,
+            application_id=meld.conf.APPLICATION_ID,
+            flags=Gio.ApplicationFlags.HANDLES_COMMAND_LINE,
         )
         GLib.set_application_name(meld.conf.APPLICATION_NAME)
         GLib.set_prgname(meld.conf.APPLICATION_ID)
@@ -73,6 +73,10 @@ class MeldApp(Gtk.Application):
             action.connect('activate', callback)
             self.add_action(action)
 
+        # Keep clipboard contents after application exit
+        clip = Gtk.Clipboard.get_default(Gdk.Display.get_default())
+        clip.set_can_store(None)
+
         self.new_window()
 
     def do_activate(self):
@@ -100,6 +104,11 @@ class MeldApp(Gtk.Application):
         self.activate()
         return 0
 
+    def do_window_removed(self, widget):
+        Gtk.Application.do_window_removed(self, widget)
+        if not len(self.get_windows()):
+            self.quit()
+
     # We can't override do_local_command_line because it has no introspection
     # annotations: https://bugzilla.gnome.org/show_bug.cgi?id=687912
 
@@ -327,8 +336,9 @@ class MeldApp(Gtk.Application):
             # TODO: support for directories specified by URIs
             file_type = f.query_file_type(Gio.FileQueryInfoFlags.NONE, None)
             if not f.is_native() and file_type == Gio.FileType.DIRECTORY:
-                raise ValueError(
-                    _("remote folder “{}” not supported").format(arg))
+                if f.get_path() is None:
+                    raise ValueError(
+                        _("remote folder “{}” not supported").format(arg))
 
             return f
 
