@@ -103,10 +103,16 @@ class MsgAreaController(Gtk.HBox):
         self.pack_start(msgarea, True, True, 0)
         return msgarea
 
-    def add_dismissable_msg(self, icon, primary, secondary):
+    def add_dismissable_msg(self, icon, primary, secondary, close_panes=None):
+        def clear_all(*args):
+            if close_panes:
+                for pane in close_panes:
+                    pane.clear()
+            else:
+                self.clear()
         msgarea = self.new_from_text_and_icon(icon, primary, secondary)
         msgarea.add_button(_("Hi_de"), Gtk.ResponseType.CLOSE)
-        msgarea.connect("response", lambda *args: self.clear())
+        msgarea.connect("response", clear_all)
         msgarea.show_all()
         return msgarea
 
